﻿using System;
using HeuristicLab.Encodings.RealVectorEncoding;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {


  /// <summary>
  /// Spacing is defined as the standarddeviation of all d[i] 
  /// where d[i] is the minimum eukildean distance any point has 
  /// to all OTHER points in the same front 
  /// </summary>
  public class Spacing : IMultiObjectiveDistance {

    public Spacing() { }


    public static double GetSpacing(RealVector[] front) {
      return new Spacing().Get(front);
    }
    public static double GetDistance(RealVector[] front, RealVector[] optimalFront) {
      return new Spacing().Get(front);
    }

    public double Compare(RealVector[] front, RealVector[] optimalFront) {
      return GetSpacing(front) - GetSpacing(optimalFront);
    }

    public double Get(RealVector[] front) {
      //TODO build a kd-tree, sort the array, do someting intelligent here
      double sum = 0;
      if (front.Length == 0) throw new Exception("Front does not contain any points");
      double[] d = new double[front.Length];
      int i = 0;
      foreach (RealVector r in front) {
        d[i] = minDistance(r, front);
        sum += d[i++];

      }
      double mean = sum / front.Length;
      sum = 0;
      foreach (double e in d) {
        sum += (e - mean) * (e - mean);
      }
      sum /= front.Length;
      return Math.Sqrt(sum);

    }

    private double minDistance(RealVector point, RealVector[] list) {
      //TODO inefficient
      double min = Double.MaxValue;
      foreach (RealVector r in list) {
        if (r == point) continue;
        double d = 0;
        for (int i = 0; i < r.Length; i++) {
          d += (point[i] - r[i]) * (point[i] - r[i]);
        }
        min = Math.Min(d, min);
      }
      return Math.Sqrt(min);
    }

  }
}
