﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("DTLZ7", "Testfunction as defined as DTLZ7 in http://repository.ias.ac.in/81671/ [30.11.15]")]
  [StorableClass]
  public class DTLZ7 : MultiObjectiveTestFunction {

    private int actualSolutionSize = 2;
    public override int ActualSolutionSize {
      get {
        return actualSolutionSize;
      }

      set {
        actualSolutionSize = value;
      }
    }

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { 0, 1 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[actualSolutionSize];
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {

        return int.MaxValue;
      }
    }

    public override int MinimumProblemSize {
      get {
        return Math.Max(2, ActualSolutionSize*10);
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    public override RealVector[] OptimalParetoFront {
      get {
        throw new NotImplementedException();
      }
    }

    public override RealVector ReferencePoint {
      get { return new RealVector(new double[] { 11.0, 11.0 }); }
    }


    [StorableConstructor]
    protected DTLZ7(bool deserializing) : base(deserializing) { }
    protected DTLZ7(DTLZ7 original, Cloner cloner) : base(original, cloner) { }
    public DTLZ7() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DTLZ7(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      return Evaluate(r, ActualSolutionSize);
    }

    private double[] Evaluate(RealVector r, int objectives) {
      if (r.Length < objectives) {
        throw new Exception("The dimensionality of the problem(ProblemSize) must be larger than or equal to ten times the dimensionality of the solution(SolutionSize) ");
      }
      double[] res = new double[objectives];

      //calculate f0...fM-1;
      double n = 10 * objectives;
      for (int i = 1; i <= objectives; i++) {
        double d = 0;
        int c = 0;
        for (int j = (int)Math.Floor((i - 1) * n / objectives); j < Math.Min((int)Math.Floor(i * n / objectives), r.Length); j++) {
          d += r[j];
          c++;
        }
        d *= 1.0 / c;
        res[i-1] = d;
      }

      //evaluate constraints g0...gM-2
      for (int i = 0; i < objectives - 1; i++) {
        if (res[objectives - 1] + 4 * res[i] - 1 < 0) return MultiObjectiveTestFunction.IllegalValue(objectives,Maximization); 
      }
      //evaluate gM-1
      double min = Double.PositiveInfinity;
      for (int i = 0; i < objectives - 1; i++) {
        for (int j = 0; j < i; j++) min = Math.Min(min, res[i] + res[j]);
      };
      if (2 * res[objectives - 1] + min - 1 < 0) return MultiObjectiveTestFunction.IllegalValue(objectives, Maximization); 

      return res;
    }
  }
}
