﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("SchafferN1", "Schaffer function N.1 for mulitobjective optimization from // https://en.wikipedia.org/wiki/Test_functions_for_optimization [30.11.2015]")]
  [StorableClass]
  public class SchafferN1 : MultiObjectiveTestFunction {

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { -1e5, 1e5 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[] { false, false };
      }
    }

    public override int MaximumProblemSize {
      get {
        return 1;
      }
    }

    public override int MaximumSolutionSize {
      get {
        return 2;
      }
    }

    public override int MinimumProblemSize {
      get {
        return 1;
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    public override int ActualSolutionSize {
      get {
        return 2;
      }

      set {
      }
    }

    public override RealVector[] OptimalParetoFront {
      get {
        return PFReader.getFromFile("SchafferN1");
      }
    }
    public override double BestKnownHypervolume {
      get {
        return new Hypervolume(base.ReferencePoint, Maximization).GetHypervolume(OptimalParetoFront);
      }
    }

    [StorableConstructor]
    protected SchafferN1(bool deserializing) : base(deserializing) { }
    protected SchafferN1(SchafferN1 original, Cloner cloner) : base(original, cloner) { }
    public SchafferN1() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SchafferN1(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      if (r.Length != 1) return null;
      double x = r[0];

      //objective1
      double f0 = x * x;

      //objective0
      double f1 = x - 2;
      f1 *= f1;

     return new double[] { f0, f1 };
    }
  }
}
